@@ -11,11 +11,11 @@ COPYRIGHT distributed with the source files.
 #include "filesend.h"
 #include "dlist.cpp"   // contains template definitions
 
-static int cq_alarm_flag = FALSE;
+static volatile bool cq_alarm_flag = false;
 
-int FilesendBuffer::open_file(const char* filename, int convert_codes) {
-  // Thefilename supplied will be opened, and convert_codes may be set to TRUE.
-  // If if is set TRUE, then $a will be translated to your call, and $b will be 
+bool FilesendBuffer::open_file(const char* filename, bool convert_codes) {
+  // Thefilename supplied will be opened, and convert_codes may be set to true.
+  // If if is set true, then $a will be translated to your call, and $b will be 
   // translated to your selcall, and $c will be translated to the call in the callsign
   // store buffer for the currently active screen
 
@@ -26,12 +26,18 @@ int FilesendBuffer::open_file(const char* filename, int convert_codes) {
         QMessageBox::warning(sendwin_ptr, "Filesend",
 				 "Cannot send a file in packet mode unless connected",
 				 QMessageBox::Ok | QMessageBox::Default, 0);
-        return FALSE;
+        return false;
     }
 
-    int return_val = FALSE;
+    bool return_val = false;
     if (file_flag == not_sending) {
-        filein.open(filename, ios::in | ios::nocreate);
+#ifdef HAVE_IOS_NOCREATE
+	filein.open(filename, ios::in | ios::nocreate);
+#else
+	// we must have Std C++ so we probably don't need a ios::nocreate
+	// flag on a read open to ensure uniqueness
+	filein.open(filename, ios::in);
+#endif
 	if (!filein) {
 	    ostrstream strm;
 	    strm << filename << " cannot be found/opened" << ends;
@@ -42,10 +48,10 @@ int FilesendBuffer::open_file(const char* filename, int convert_codes) {
 	}
 	else {
 	    file_flag = loading_buffer;
-	    return_val = TRUE;
+	    return_val = true;
 	    if (get_buffer_mode() == FileBuffer::text) convert_codes_flag = convert_codes;
 	    else {
-	        convert_codes_flag = FALSE;
+	        convert_codes_flag = false;
 		receivewin_ptr->write("\nSending file ...\n", get_stream(), get_port());
 	    }
 	}
@@ -53,7 +59,7 @@ int FilesendBuffer::open_file(const char* filename, int convert_codes) {
     return return_val;
 }
 
-int FilesendBuffer::load_buffer(void) {
+bool FilesendBuffer::load_buffer(void) {
 
   // if we were sending a file and now it has all gone to the Kam, signal this
   // (when this method has finished loading the file into tr_buffer, we set
@@ -64,31 +70,38 @@ int FilesendBuffer::load_buffer(void) {
   // this method should accordingly be called by the program timer event handler whenever
   // FilesendBuffer::file_flag != FilesendBuffer::not_sending
 
-  // the program returns FALSE when FilesendBuffer::file_flag == FilesendBuffer::not_sending,
-  // otherwise it returns TRUE
+  // the program returns false when FilesendBuffer::file_flag == FilesendBuffer::not_sending,
+  // or if FilesendBuffer::file_flag == FilesendBuffer::loading_complete and the buffer is empty,
+  // otherwise it returns true
 
-    if (file_flag == not_sending) return FALSE;
+  // the method FilesendBuffer::end_loading() is available to put the FilesendBuffer object
+  // into a state which will itself and will ensure that FilesendBuffer::load_buffer()
+  // will return false so that it may be safely deleted by the function/method which
+  // calls FilesendBuffer::end_loading
+
+    if (file_flag == not_sending) return false;
     if (file_flag == loading_complete) {
-        if (Qqueue<char>::is_empty()) {
+        if (Iqueue<char>::is_empty()) {
 	    if (get_buffer_mode() == FileBuffer::binary || get_buffer_mode() == FileBuffer::s_plus) {
 		receivewin_ptr->write("\nSending completed.\n", get_stream(), get_port());
 	    }
 	    file_flag = not_sending;
+	    filein.clear();
 	    filein.close();
-	    return FALSE;
+	    return false;
 	}
     }
 
     char letter;
     int result = 0;
     int count;
-    int write_to_sendwin_flag = FALSE;
+    bool write_to_sendwin_flag = false;
     int sendwin_count = 0;
 
     if (get_stream() == tnc_ptr->tnc_func.active_stream()
 	&& get_port() == tnc_ptr->tnc_func.active_port
 	&& get_buffer_mode() == FileBuffer::text) {
-        write_to_sendwin_flag = TRUE;
+        write_to_sendwin_flag = true;
     }
 
     for (count = 0; result != -1 && count < TR_BUFFER_SIZE/2 && filein.get(letter); count++) {
@@ -104,12 +117,12 @@ int FilesendBuffer::load_buffer(void) {
 		    && (tnc_ptr->tnc_func.stream_status[0][1] == Tnc_func::disconnected
 			|| tnc_ptr->tnc_func.tor_connected_mode == Tnc_func::not_pactor_gtor))) {
 	        if (!is_baudot(letter)) letter = 0;
-		else if (tnc_ptr->tnc_func.hfmode == Tnc_func::rtty) letter = toupper(letter);
+		else if (tnc_ptr->tnc_func.hfmode == Tnc_func::rtty) letter = toupper(static_cast<unsigned char> (letter));
 	    }
 	    else if (tnc_ptr->tnc_func.hfmode == Tnc_func::cw) {
 	        if (!is_morse(letter) 
 		     && !(letter == '$' && convert_codes_flag)) letter = 0; // allow the $ token through
-		else letter = toupper(letter);
+		else letter = toupper(static_cast<unsigned char> (letter));
 	    }
 	}
 
@@ -126,14 +139,14 @@ int FilesendBuffer::load_buffer(void) {
 	    if (letter == '$' && convert_codes_flag) {
 	        int second_letter = filein.peek();
 		if (second_letter == 'a') {
-		    if ((unsigned int)is_free() >= strlen(prog_func.myCall)) {
-		        for (char* letter_ptr = prog_func.myCall; *letter_ptr; letter_ptr++) {
+		    if ((string::size_type)is_free() >= prog_func.myCall.size()) {
+		        for (const char* letter_ptr = prog_func.myCall.c_str(); *letter_ptr; letter_ptr++) {
 			    add_item(*letter_ptr);
 			}
 			if (write_to_sendwin_flag) {
 			    sendwin_buffer[sendwin_count] = 0; // null terminate string
 			    sendwin_ptr->write(sendwin_buffer);
-			    sendwin_ptr->write(prog_func.myCall);
+			    sendwin_ptr->write(prog_func.myCall.c_str());
 			    sendwin_count = 0;
 			}
 			second_letter = filein.get(); // get rid of the 'a' code
@@ -144,14 +157,14 @@ int FilesendBuffer::load_buffer(void) {
 		    }
 		}
 		else if (second_letter == 'b') {
-		    if ((unsigned int)is_free() >= strlen(prog_func.mySelCall)) {
-		        for (char* letter_ptr = prog_func.mySelCall; *letter_ptr; letter_ptr++) {
+		    if ((string::size_type)is_free() >= prog_func.mySelCall.size()) {
+		        for (const char* letter_ptr = prog_func.mySelCall.c_str(); *letter_ptr; letter_ptr++) {
 			    add_item(*letter_ptr);
 			}
 			if (write_to_sendwin_flag) {
 			    sendwin_buffer[sendwin_count] = 0; // null terminate string
 			    sendwin_ptr->write(sendwin_buffer);
-			    sendwin_ptr->write(prog_func.mySelCall);
+			    sendwin_ptr->write(prog_func.mySelCall.c_str());
 			    sendwin_count = 0;
 			}
 			second_letter = filein.get(); // get rid of the 'b' code
@@ -162,15 +175,15 @@ int FilesendBuffer::load_buffer(void) {
 		    }
 		}
 		else if (second_letter == 'c') {
-		    if ((unsigned int)is_free() >= 
-			   strlen(tnc_ptr->tnc_func.hisCall[tnc_ptr->tnc_func.active_stream()][tnc_ptr->tnc_func.active_port])) {
-		        for (char* letter_ptr = tnc_ptr->tnc_func.hisCall[tnc_ptr->tnc_func.active_stream()][tnc_ptr->tnc_func.active_port]; *letter_ptr; letter_ptr++) {
+		    if ((string::size_type)is_free() >= 
+			   tnc_ptr->tnc_func.hisCall[tnc_ptr->tnc_func.active_stream()][tnc_ptr->tnc_func.active_port].size()) {
+		        for (const char* letter_ptr = tnc_ptr->tnc_func.hisCall[tnc_ptr->tnc_func.active_stream()][tnc_ptr->tnc_func.active_port].c_str(); *letter_ptr; letter_ptr++) {
 			    add_item(*letter_ptr);
 			}
 			if (write_to_sendwin_flag) {
 			    sendwin_buffer[sendwin_count] = 0; // null terminate string
 			    sendwin_ptr->write(sendwin_buffer);
-			    sendwin_ptr->write(tnc_ptr->tnc_func.hisCall[tnc_ptr->tnc_func.active_stream()][tnc_ptr->tnc_func.active_port]);
+			    sendwin_ptr->write(tnc_ptr->tnc_func.hisCall[tnc_ptr->tnc_func.active_stream()][tnc_ptr->tnc_func.active_port].c_str());
 			    sendwin_count = 0;
 			}
 			second_letter = filein.get(); // get rid of the 'c' code
@@ -252,6 +265,29 @@ int FilesendBuffer::load_buffer(void) {
 			result = -1;
 		    }
 		}
+		else if (second_letter == 'g') {
+		    ostrstream s1;
+		    s1 << setfill('0') << setw(3) << prog_func.qso_count << ends;
+		    char* qso_count_string = s1.str();
+		    
+		    if ((unsigned int)is_free() >= strlen(qso_count_string)) {
+			for (char* letter_ptr = qso_count_string; *letter_ptr; letter_ptr++) {
+			    add_item(*letter_ptr);
+			}
+			if (write_to_sendwin_flag) {
+			    sendwin_buffer[sendwin_count] = 0; // null terminate string
+			    sendwin_ptr->write(sendwin_buffer);
+			    sendwin_ptr->write(qso_count_string);
+			    sendwin_count = 0;
+			}
+			second_letter = filein.get(); // get rid of the 'g' code
+		    }
+		    else {           // not sufficient room in buffer for qso count
+		        filein.putback('$');
+			result = -1;
+		    }
+		    delete[] qso_count_string;
+		}
 		else if (second_letter == EOF) {  // last letter of file is a '$'
 		    result = add_item('$');
 		    if (result == -1) filein.putback('$');
@@ -295,23 +331,28 @@ int FilesendBuffer::load_buffer(void) {
     else if (file_flag == loading_buffer && !filein) {
         file_flag = loading_complete;
     }
-    return TRUE;
+    return true;
 }
 
-char FilesendBuffer::front(Qqueue_enum::extract_mode mode) {
-    char return_val = Qqueue<char>::front(mode);
-    if (mode == Qqueue_enum::remove) bytes_sent++;
+char FilesendBuffer::front(Extract_mode mode) {
+    char return_val = Iqueue<char>::front(mode);
+    if (mode == remove) bytes_sent++;
     return return_val;
 }
 
-int CqsendBuffer::load_buffer(void) {
-  // this method should accordingly be called by the program timer event handler whenever
+bool CqsendBuffer::load_buffer(void) {
+  // this method should be called by the program timer event handler whenever
   // an object of this class is in existence
   // so that the buffer is properly flushed and the cq alarm is properly set
 
-  // the program returns FALSE when this is the last call the CqsendBuffer::load_buffer(void),
+  // the program returns false when this is the last call the CqsendBuffer::load_buffer(void),
   // or if the cq file cannot be opened
-  // otherwise it returns TRUE
+  // otherwise it returns true
+
+  // the method CqsendBuffer::end_loading() is available to put the CqsendBuffer object
+  // into a state which will itself and will ensure that CqsendBuffer::load_buffer()
+  // will return false so that it may be safely deleted by the function/method which
+  // calls CqsendBuffer::end_loading
 
   // only this method is intended to write to prog_func.sending_autocq, although
   // prog_func.sending_autocq is intended to be generally readable by other
@@ -319,15 +360,16 @@ int CqsendBuffer::load_buffer(void) {
 
 
     if (autocq_flag == off) {
-        char* filename = new char[strlen(prog_func.filedir) + 4];
-	if (!filename) {
-	    cerr << "Memory allocation error in CqsendBuffer::load_buffer()" << endl;
-	    exit(MEM_ERROR);
-	}
-	strcpy(filename, prog_func.filedir);
-	strcat(filename, "/cq");
+        string filename(prog_func.filedir);
+	filename += "/cq";
     
-	filein.open(filename, ios::in | ios::nocreate);
+#ifdef HAVE_IOS_NOCREATE
+	filein.open(filename.c_str(), ios::in | ios::nocreate);
+#else
+	// we must have Std C++ so we probably don't need a ios::nocreate
+	// flag on a read open to ensure uniqueness
+	filein.open(filename.c_str(), ios::in);
+#endif
 	if (!filein) {
 	    ostrstream strm;
 	    strm << filename << " cannot be found/opened" << ends;
@@ -335,8 +377,8 @@ int CqsendBuffer::load_buffer(void) {
 	    QMessageBox::warning(sendwin_ptr, "Auto-CQ", message,
 				 QMessageBox::Ok | QMessageBox::Default, 0);
 	    delete[] message;
-	    prog_func.sending_autocq = FALSE;
-	    return FALSE;
+	    prog_func.sending_autocq = false;
+	    return false;
 	}
 	else {
 	    if (tnc_ptr->tnc_func.hfmode == Tnc_func::tor
@@ -347,22 +389,21 @@ int CqsendBuffer::load_buffer(void) {
 		usleep(200000);
 	    }
 	    autocq_flag = loading_cq_buffer;
-	    prog_func.sending_autocq = TRUE;
+	    prog_func.sending_autocq = true;
 	    mainscreen_ptr->display_connected_status();
 	    mainscreen_ptr->set_auto_cq_button();
 	    tnc_ptr->send_specialcommand(txCMD);
 	}
-	delete[] filename;
     }
 
-    int return_val = TRUE;
+    bool return_val = true;
     char letter;
     int result = 0;
     int count;
-    int write_to_sendwin_flag = FALSE;
+    bool write_to_sendwin_flag = false;
     int sendwin_count = 0;
 
-    if (tnc_ptr->tnc_func.active_port) write_to_sendwin_flag = TRUE;
+    if (tnc_ptr->tnc_func.active_port) write_to_sendwin_flag = true;
 
     if (autocq_flag == loading_cq_buffer && tnc_ptr->tnc_func.tx_status == Tnc_func::tx) {
         for (count = 0; result != -1 && count < CQ_BUFFER_SIZE/2 && filein.get(letter); count++) {
@@ -381,15 +422,15 @@ int CqsendBuffer::load_buffer(void) {
 #endif
 	        if (letter == '$') {
 		    int second_letter = filein.peek();
-		        if (second_letter == 'a') {
-			    if ((unsigned int)is_free() >= strlen(prog_func.myCall)) {
-			        for (char* letter_ptr = prog_func.myCall; *letter_ptr; letter_ptr++) {
-				add_item(*letter_ptr);
+		    if (second_letter == 'a') {
+		        if ((string::size_type)is_free() >= prog_func.myCall.size()) {
+			    for (const char* letter_ptr = prog_func.myCall.c_str(); *letter_ptr; letter_ptr++) {
+			        add_item(*letter_ptr);
 			    }
 			    if (write_to_sendwin_flag) {
 			        sendwin_buffer[sendwin_count] = 0; // null terminate string
 				sendwin_ptr->write(sendwin_buffer);
-				sendwin_ptr->write(prog_func.myCall);
+				sendwin_ptr->write(prog_func.myCall.c_str());
 				sendwin_count = 0;
 			    }
 			    second_letter = filein.get(); // get rid of the 'a' code
@@ -400,14 +441,14 @@ int CqsendBuffer::load_buffer(void) {
 			}
 		    }
 		    else if (second_letter == 'b') {
-		        if ((unsigned int)is_free() >= strlen(prog_func.mySelCall)) {
-			    for (char* letter_ptr = prog_func.mySelCall; *letter_ptr; letter_ptr++) {
+		        if ((string::size_type)is_free() >= prog_func.mySelCall.size()) {
+			    for (const char* letter_ptr = prog_func.mySelCall.c_str(); *letter_ptr; letter_ptr++) {
 			        add_item(*letter_ptr);
 			    }
 			    if (write_to_sendwin_flag) {
 			        sendwin_buffer[sendwin_count] = 0; // null terminate string
 				sendwin_ptr->write(sendwin_buffer);
-				sendwin_ptr->write(prog_func.mySelCall);
+				sendwin_ptr->write(prog_func.mySelCall.c_str());
 				sendwin_count = 0;
 			    }
 			    second_letter = filein.get(); // get rid of the 'b' code
@@ -418,15 +459,15 @@ int CqsendBuffer::load_buffer(void) {
 			}
 		    }
 		    else if (second_letter == 'c') {
-		        if ((unsigned int)is_free() >= 
-			      strlen(tnc_ptr->tnc_func.hisCall[0][1])) {
-			    for (char* letter_ptr = tnc_ptr->tnc_func.hisCall[0][1]; *letter_ptr; letter_ptr++) {
+		        if ((string::size_type)is_free() >= 
+			    tnc_ptr->tnc_func.hisCall[0][1].size()) {
+			    for (const char* letter_ptr = tnc_ptr->tnc_func.hisCall[0][1].c_str(); *letter_ptr; letter_ptr++) {
 			        add_item(*letter_ptr);
 			    }
 			    if (write_to_sendwin_flag) {
 			        sendwin_buffer[sendwin_count] = 0; // null terminate string
 				sendwin_ptr->write(sendwin_buffer);
-				sendwin_ptr->write(tnc_ptr->tnc_func.hisCall[0][1]);
+				sendwin_ptr->write(tnc_ptr->tnc_func.hisCall[0][1].c_str());
 				sendwin_count = 0;
 			    }
 			    second_letter = filein.get(); // get rid of the 'c' code
@@ -508,6 +549,29 @@ int CqsendBuffer::load_buffer(void) {
 			    result = -1;
 			}
 		    }
+		    else if (second_letter == 'g') {
+		        ostrstream s1;
+			s1 << setfill('0') << setw(3) << prog_func.qso_count << ends;
+			char* qso_count_string = s1.str();
+		    
+			if ((unsigned int)is_free() >= strlen(qso_count_string)) {
+			    for (char* letter_ptr = qso_count_string; *letter_ptr; letter_ptr++) {
+			        add_item(*letter_ptr);
+			    }
+			    if (write_to_sendwin_flag) {
+			        sendwin_buffer[sendwin_count] = 0; // null terminate string
+				sendwin_ptr->write(sendwin_buffer);
+				sendwin_ptr->write(qso_count_string);
+				sendwin_count = 0;
+			    }
+			    second_letter = filein.get(); // get rid of the 'g' code
+			}
+			else {           // not sufficient room in buffer for qso count
+			    filein.putback('$');
+			    result = -1;
+			}
+			delete[] qso_count_string;
+		    }
 		    else if (second_letter == EOF) {  // last letter of file is a '$'
 		        result = add_item('$');
 			if (result == -1) filein.putback('$');
@@ -556,7 +620,7 @@ int CqsendBuffer::load_buffer(void) {
 	    filein.clear();               // go back to begining of file ready for next pass through
 	    filein.seekg(0, ios::beg);    // after the timer alarm goes off
 	    autocq_flag = waiting;
-	    cq_alarm_flag = FALSE;
+	    cq_alarm_flag = false;
 	    signal(SIGALRM, cq_alarm_handler);
 	    int alarm_val = prog_func.autocq_delay;
 	    if (!alarm_val) alarm_val = 90;
@@ -564,11 +628,15 @@ int CqsendBuffer::load_buffer(void) {
 	    rx_count = 0;
 	    if (tnc_ptr->tnc_func.hfmode == Tnc_func::tor
 		  && prog_func.tor_autocq_mode == Prog_func::pactor) {
-// we need to go back to Tor Standby mode if we having been sending FEC in Pactor
-// Note: if we cancel a cq message half way through by means of Autocq_query::action(),
-// Autocq_query::action() will set prog_func.sending_autocq to FALSE, with the result
-// that Tnc::read_state_message() will automatically do the same, so there is no
-// to cater here for that possibility
+// we need to go back to Tor Standby mode if we have been sending FEC
+// in Pactor.  Note: if we cancel a cq message half way through by means
+// of EventSlots::auto_cq(), EventSlots::auto_cq() will (by calling
+// CqsendBuffer::end_loading()) set autocq_flag to
+// CqsendBuffer::close_down which will in turn cause this method to
+// set prog_func.sending_autocq to false, with the result that
+// Tnc::read_state_message() will automatically change the TNC back to Tor
+// Standby mode, so there is no need to cater here for a change of mode
+// in those circumstances
 	        tnc_ptr->send_specialcommand(packetCMD);
 		usleep(300000);
 		tnc_ptr->send_kamcommand(torCMD, '2', '0');
@@ -577,13 +645,18 @@ int CqsendBuffer::load_buffer(void) {
 	}
     }
     else if (autocq_flag == waiting) {
-        if (tnc_ptr->tnc_func.stream_status[0][1] ==  Tnc_func::connected) {
+        // if we have made a connection or we have had a slip-up in changing from
+        // Pactor mode to Tor mode when sending a Tor mode autocq using Pactor FEC
+        // then we need to end the autocq
+        if (tnc_ptr->tnc_func.stream_status[0][1] ==  Tnc_func::connected
+	    || tnc_ptr->tnc_func.hfmode == Tnc_func::packet) {
+	    filein.clear();
 	    filein.close();
-	    prog_func.sending_autocq = FALSE;
+	    prog_func.sending_autocq = false;
 	    mainscreen_ptr->display_connected_status();
 	    // set_auto_cq_button() will be called in MainScreenFuncs::timerEvent()
 	    // once the buffer is extracted - calling it now will give a false display
-	    return_val = FALSE;
+	    return_val = false;
 	}
 	else if (cq_alarm_flag) {
 	    QApplication::beep();
@@ -596,23 +669,24 @@ int CqsendBuffer::load_buffer(void) {
 		usleep(100000);
 	    }
 	    tnc_ptr->send_specialcommand(txCMD);  // go back to transmit
-	    cq_alarm_flag = FALSE;
+	    cq_alarm_flag = false;
 	}
     }
     else if (autocq_flag == close_down) {
+	filein.clear();
         filein.close();
-	prog_func.sending_autocq = FALSE;
+	prog_func.sending_autocq = false;
 	mainscreen_ptr->display_connected_status();
 	// set_auto_cq_button() will be called in MainScreenFuncs::timerEvent()
 	// once the buffer is extracted - calling it now will give a false display
-	return_val = FALSE;
+	return_val = false;
     }
     return return_val;
 }
 
 
 void cq_alarm_handler(int) {
-    cq_alarm_flag = TRUE;
+    cq_alarm_flag = true;
 }
 
 BufferList::BufferList(void) {
